/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.se.eip.patterns;

import java.util.List;
import java.util.logging.Level;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;

import org.ow2.petals.component.framework.Constants.MEPConstants;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.jbi.descriptor.Consumes;
import org.ow2.petals.se.eip.ExchangeContext;

/**
 *
 * @author alouis
 *
 */
public class Dispatcher implements Pattern{

	/**
	 * Dispatch the exchange to all Consumes services defined in the service unit. <br>
	 * <br> - Accept InOnly and RobustInOnly
	 * <br> - If a Fault or Error occurs on a call, stop the process and return the Fault or Error
	 * Note : The exchange will be sent back automatically by the framework
	 *
	 * @param exchange
	 */
	public void processPattern(Exchange exchange, ExchangeContext context) {
		context.getLogger().info("dispatch");
		Exchange originalExchange = exchange;
		if (processMEPValidation(exchange, context)) {

			List<Consumes> consumesList = context
					.getSUConsumes(originalExchange.getEndpoint());

			sendAsyncExchanges(consumesList,exchange, context);
		}
	}

	/**
	 * Check InOnly or RobustInOnly pattern. <br>
	 * Set a fault on the exchange if the pattern is different, and return
	 * false.
	 *
	 * @param exchange
	 * @param context
	 * @return true if MEP is InOnly or RobustInOnly, false otherwise
	 */
	protected boolean processMEPValidation(Exchange exchange,
			ExchangeContext context) {
		if (exchange.isInOutPattern() || exchange.isInOptionalOutPattern()) {
			String fault = "The Dispatch pattern only works for InOnly or RobustInOnly patterns.";
			context.getLogger().warning(fault);
			try {
				exchange.setFault(new Exception(fault));
			} catch (MessagingException e) {
				context.getLogger().severe(e.getMessage());
				exchange.setError(e);
			}
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Send a copy of the exchange's IN message to all the services to consume.
	 * If an error occurs, set a Fault on the original exchange (for robust exchange). MEP used is InOnly
	 * @param consumesList
	 * @param exchange
	 * @param context
	 */
	protected void sendAsyncExchanges(List<Consumes> consumesList,
			Exchange exchange, ExchangeContext context) {

		try {
			NormalizedMessage inMessage = exchange.getInMessage();
			// send all exchanges asynchronously, inonly
			for (Consumes consumedService : consumesList) {
				Exchange tmpExchange = PatternHelper.createExchange(
						consumedService, MEPConstants.IN_ONLY_PATTERN.value(),
						inMessage, context);
				context.getLogger().info(
						"send exchange to ...\n - itf="
								+ consumedService.getInterfaceName()
								+ "\n - srv="
								+ consumedService.getServiceName()
								+ "\n - ep ="
								+ consumedService.getEndpointName());
				context.sendAsync(tmpExchange);
			}
		} catch (JBIException e) {
			context.getLogger().log(Level.SEVERE, e.getMessage(), e);
			try {
				exchange.setFault(e);
			} catch (MessagingException ex) {
				context.getLogger().severe(ex.getMessage());
				exchange.setError(ex);
			}
		}
	}
}
