/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.eip.patterns;

import java.net.URI;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.JBIException;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;

import org.ow2.petals.component.framework.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.component.framework.util.SourceHelper;
import org.ow2.petals.jbi.descriptor.Consumes;
import org.ow2.petals.se.eip.ExchangeContext;
/**
 *
 * @author alouis
 *
 */
public class PatternHelper {
	/**
	 * Process the result of a piped exchange.<br> - if the exchange contains
	 * an OUT response, return it and closed the piped exchange<br> - if the
	 * exchange contains a FAULT, set it on the original exchange and return
	 * nothing, piped exchange is closed<br> - if the exchange is ERROR, set it
	 * on the original exchange and return nothing<br> - if the exchange is
	 * DONE, return nothing<br>
	 *
	 * @param sentExchange
	 * @param originalExchange
	 * @param context
	 * @return the OUT message of the piped exchange, if any. When null, the
	 *         original exchange has been updated
	 * @throws JBIException
	 */
	public static NormalizedMessage processSentExchange(Exchange sentExchange,
			Exchange originalExchange, ExchangeContext context) throws JBIException {
		NormalizedMessage receivedMessage = null;
		context.getLogger().info(
				"process exchange received from...\n - srv="
						+ sentExchange.getEndpoint().getServiceName()
						+ "\n - ep ="
						+ sentExchange.getEndpoint().getEndpointName());
		if (sentExchange.isActiveStatus()) {
			if (sentExchange.getFault() != null) {
				context.getLogger()
						.info(
								"a fault is returned, set it on the original Exchange.");
				// fault on piped exchange, set it on the original exchange
				// do not return a message
				Fault fault = originalExchange.createFault();
				fault.setContent(sentExchange.getFault().getContent());
			} else {
				// out message received, return it
				context.getLogger().info("an Out message is returned.");
				receivedMessage = sentExchange.getOutMessage();
			}
			// close the exchange with the piped service
			sentExchange.setStatus(ExchangeStatus.DONE);
			context.send(sentExchange);
		} else if (sentExchange.isErrorStatus()) {
			// error on piped exchange, set it on the original exchange
			// do not return a message
			context.getLogger()
					.info(
							"the exchange status is Error, set the original Exchange status to Error.");
			originalExchange.setError(sentExchange.getError());
		} else if (sentExchange.isDoneStatus()) {
			// piped service return nothing, exchange is closed
			// do not return a message
			context.getLogger().info("the exchange status is Done, do nothing.");
		}

		return receivedMessage;
	}

	/**
	 * Create an exchange for the given Consumes services. Set the IN message
	 * with a <em>copy</em> of the given message
	 *
	 * @param consumes
	 * @param mep
	 * @param originalMessage
	 * @return
	 * @throws JBIException
	 * @throws PEtALSCDKException
	 */
	public static Exchange createExchange(Consumes consumes, URI mep,
			NormalizedMessage originalMessage, ExchangeContext context) throws JBIException,
			PEtALSCDKException {
		Exchange tmpExchange = context.createConsumeExchange(consumes, mep);
		NormalizedMessage tmpMessage = tmpExchange.getInMessage();
		copy(originalMessage, tmpMessage);
		return tmpExchange;

	}

	/**
	 * Copies a message's content, properties and attachments in another.
	 *
	 * @param originalNm
	 * @param copyNm
	 * @throws MessagingException
	 *             error while setting attachment
	 * @throws PEtALSCDKException
	 */
	@SuppressWarnings("unchecked")
	public  static void copy(final NormalizedMessage originalNm,
			final NormalizedMessage copyNm) throws MessagingException,
			PEtALSCDKException {
		// setting content, with hardcopy (because if semantic routing...)
		final String contentString = SourceHelper.createString(originalNm
				.getContent());
		copyNm.setContent(SourceHelper.createSource(contentString));

		// setting properties
		final Set<String> propertyNames = originalNm.getPropertyNames();
		for (final String propertyName : propertyNames) {
			copyNm.setProperty(propertyName, originalNm
					.getProperty(propertyName));
		}

		// setting attachments if any
		final Set<String> inAttachmentNameSet = originalNm.getAttachmentNames();
		if (inAttachmentNameSet != null) {
			for (final String attachmentName : inAttachmentNameSet) {
				final DataHandler attDh = originalNm
						.getAttachment(attachmentName);
				copyNm.addAttachment(attachmentName, attDh);
			}
		}
	}



}
