/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.eip.patterns;

import java.util.List;
import java.util.logging.Level;

import javax.jbi.JBIException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.ow2.petals.component.framework.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.jbi.descriptor.Consumes;
import org.ow2.petals.se.eip.ExchangeContext;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 *
 * @author alouis
 *
 */
public class Router implements Pattern {

	public static final String CONDITION = "condition";

	public static final XPath XPATH;
	static {
		// initialize static fields
		XPATH = XPathFactory.newInstance().newXPath();
	}

	/**
	 * Route the exchange on the correct Consumes services defined in the
	 * service unit. <br>
	 * Return the result of the service, if any.<br>
	 *
	 * @param exchange
	 */
	public void processPattern(Exchange exchange, ExchangeContext context) {
		context.getLogger().info("router");

		try {
			List<Consumes> consumesList = context.getSUConsumes(exchange
					.getEndpoint());
			// there must be only 2 'consumes' defined in the SU
			int count = consumesList.size();
			if (count <= 0 || count > 2) {
				throw new PEtALSCDKException(
						"only 2 <Consumes> elements must be defined in the service-unit.");
			} else {
				Exchange originalExchange = exchange;
				// retrieve the condition
				String condition = context.getExtensions().getValue(CONDITION);
				// evaluate the IN message
				Document inMessage = originalExchange
						.getInMessageContentAsDocument();
				boolean eval = evaluateCondition(condition, inMessage
						.getFirstChild(), context);
				// forward the exchange to the correct service, depending on the
				// test result
				Consumes consumedService = consumesList.get(eval ? 0 : 1);
				Exchange routedExchange = PatternHelper.createExchange(
						consumedService, originalExchange.getPattern(),
						originalExchange.getInMessage(), context);
				context.getLogger().info(
						"send exchange to ...\n - itf="
								+ consumedService.getInterfaceName()
								+ "\n - srv="
								+ consumedService.getServiceName()
								+ "\n - ep ="
								+ consumedService.getEndpointName());
				context.sendSync(routedExchange);

				// process the response
				NormalizedMessage receivedMessage = PatternHelper
						.processSentExchange(routedExchange, originalExchange,
								context);
				// if a response has been returned, set it on the original
				// exchange
				// otherwise, the 'processSentExchange()' has manage the
				// exchange
				if (receivedMessage != null) {
					originalExchange.setOutMessage(receivedMessage);
				}
			}
		} catch (JBIException e) {
			context.getLogger().log(Level.SEVERE, e.getMessage(), e);
			exchange.setError(e);
		} catch (XPathExpressionException e) {
			context.getLogger().log(Level.SEVERE, e.getMessage(), e);
			exchange.setError(e);
		}
	}

	/**
	 * Evaluate the given XPATH condition on the Node
	 *
	 * @param condition
	 * @param message
	 * @return
	 * @throws XPathExpressionException
	 */
	private boolean evaluateCondition(String condition, final Node message,
			ExchangeContext context) throws XPathExpressionException {

		try {
			final XPathExpression expression = XPATH.compile(condition);
			final String conditionResult = expression.evaluate(message);

			context.getLogger().info(
					"Evaluate condition: \"" + condition + "\", result: \""
							+ conditionResult + "\".");

			return conditionResult.equals("true");

		} catch (final XPathExpressionException e) {
			final String err = "Error in XPathExpression \"" + condition
					+ "\", cause: \"" + e.getCause().getMessage() + "\".";
			context.getLogger().log(Level.SEVERE, err);
			throw e;
		}
	}
}
